@@ -692,14 +692,14 @@ void F4GB::insert_gb_element(row_elem &r)
       int *exp = newarray_atomic(int, M->n_vars());
       M->to_intstar_vector(result->f.monoms, exp, x);
       hilbert->addMonomial(exp, x + 1);
-      deletearray(exp);
+      freemem(exp);
     }
 
   // now insert the lead monomial into the lookup table
   varpower_monomial vp = newarray_atomic(varpower_word, 2 * M->n_vars() + 1);
   M->to_varpower_monomial(result->f.monoms, vp);
   lookup->insert_minimal_vp(M->get_component(result->f.monoms), vp, which);
-  deleteitem(vp);
+  freemem(vp);
   // now go forth and find those new pairs
   S->find_new_pairs(is_ideal);
 }
