@@ -12,6 +12,10 @@
 
 #include <M2/gc-include.h>
 
+#ifdef MEMDEBUG
+#include "memdebug.h"
+#endif
+
 static volatile AO_t monideals_nfinalized = 0;
 static volatile AO_t monideals_nremoved = 0;
 
@@ -36,6 +40,9 @@ static volatile AO_t schorder_nremoved = 0;
 //////////////////////////////////////////////////////
 extern "C" void remove_monideal(void *p, void *cd)
 {
+#ifdef MEMDEBUG
+  p = M2_debug_to_inner(p);
+#endif
   MonomialIdeal *G = static_cast<MonomialIdeal *>(p);
   AO_t nremoved = AO_fetch_and_add1(&monideals_nremoved);
   if (M2_gbTrace >= 3)
@@ -44,7 +51,11 @@ extern "C" void remove_monideal(void *p, void *cd)
 }
 void intern_monideal(MonomialIdeal *G)
 {
-  GC_REGISTER_FINALIZER(G, remove_monideal, 0, 0, 0);
+#ifdef MEMDEBUG
+  GC_REGISTER_FINALIZER(M2_debug_to_outer(G), remove_monideal, 0, 0, 0);
+#else
+  GC_REGISTER_FINALIZER(G,                    remove_monideal, 0, 0, 0);
+#endif
   AO_t nfinalized = AO_fetch_and_add1(&monideals_nfinalized);
   if (M2_gbTrace >= 3)
     fprintf(stderr,
@@ -55,6 +66,9 @@ void intern_monideal(MonomialIdeal *G)
 //////////////////////////////////////////////////////
 extern "C" void remove_polyring(void *p, void *cd)
 {
+#ifdef MEMDEBUG
+  p = M2_debug_to_inner(p);
+#endif
   PolynomialRing *G = static_cast<PolynomialRing *>(p);
   AO_t nremoved = AO_fetch_and_add1(&polyrings_nremoved);
   if (M2_gbTrace >= 3)
@@ -63,8 +77,11 @@ extern "C" void remove_polyring(void *p, void *cd)
 }
 void intern_polyring(const PolynomialRing *G)
 {
-  GC_REGISTER_FINALIZER_IGNORE_SELF(
-      const_cast<PolynomialRing *>(G), remove_polyring, 0, 0, 0);
+#ifdef MEMDEBUG
+  GC_REGISTER_FINALIZER_IGNORE_SELF(M2_debug_to_outer(const_cast<PolynomialRing *>(G)), remove_polyring, 0, 0, 0);
+#else
+  GC_REGISTER_FINALIZER_IGNORE_SELF(                  const_cast<PolynomialRing *>(G) , remove_polyring, 0, 0, 0);
+#endif
   AO_t nfinalized = AO_fetch_and_add1(&polyrings_nfinalized);
   if (M2_gbTrace >= 3)
     fprintf(stderr,
@@ -75,6 +92,9 @@ void intern_polyring(const PolynomialRing *G)
 //////////////////////////////////////////////////////
 extern "C" void remove_gb(void *p, void *cd)
 {
+#ifdef MEMDEBUG
+  p = M2_debug_to_inner(p);
+#endif
   GBComputation *G = static_cast<GBComputation *>(p);
   AO_t nremoved = AO_fetch_and_add1(&gbs_nremoved);
   if (M2_gbTrace >= 3)
@@ -83,8 +103,11 @@ extern "C" void remove_gb(void *p, void *cd)
 }
 void intern_GB(GBComputation *G)
 {
-  //  GC_REGISTER_FINALIZER(G,remove_gb,0,0,0);
-  GC_REGISTER_FINALIZER_IGNORE_SELF(G, remove_gb, 0, 0, 0);
+#ifdef MEMDEBUG
+  GC_REGISTER_FINALIZER_IGNORE_SELF(M2_debug_to_outer(G), remove_gb, 0, 0, 0);
+#else
+  GC_REGISTER_FINALIZER_IGNORE_SELF(                  G , remove_gb, 0, 0, 0);
+#endif
   AO_t nfinalized = AO_fetch_and_add1(&gbs_nfinalized);
   if (M2_gbTrace >= 3)
     {
@@ -99,6 +122,9 @@ void intern_GB(GBComputation *G)
 //////////////////////////////////////////////////////
 extern "C" void remove_res(void *p, void *cd)
 {
+#ifdef MEMDEBUG
+  p = M2_debug_to_inner(p);
+#endif
   ResolutionComputation *G = static_cast<ResolutionComputation *>(p);
   AO_t nremoved = AO_fetch_and_add1(&res_nremoved);
   if (M2_gbTrace >= 3)
@@ -107,7 +133,11 @@ extern "C" void remove_res(void *p, void *cd)
 }
 void intern_res(ResolutionComputation *G)
 {
-  GC_REGISTER_FINALIZER(G, remove_res, 0, 0, 0);
+#ifdef MEMDEBUG
+  GC_REGISTER_FINALIZER(M2_debug_to_outer(G), remove_res, 0, 0, 0);
+#else
+  GC_REGISTER_FINALIZER(                  G , remove_res, 0, 0, 0);
+#endif
   AO_t nfinalized = AO_fetch_and_add1(&res_nfinalized);
   if (M2_gbTrace >= 3)
     fprintf(
@@ -116,6 +146,9 @@ void intern_res(ResolutionComputation *G)
 //////////////////////////////////////////////////////
 extern "C" void remove_SchreyerOrder(void *p, void *cd)
 {
+#ifdef MEMDEBUG
+  p = M2_debug_to_inner(p);
+#endif
   SchreyerOrder *G = static_cast<SchreyerOrder *>(p);
   AO_t nremoved = AO_fetch_and_add1(&schorder_nremoved);
   if (M2_gbTrace >= 3)
@@ -124,7 +157,11 @@ extern "C" void remove_SchreyerOrder(void *p, void *cd)
 }
 void intern_SchreyerOrder(SchreyerOrder *G)
 {
-  GC_REGISTER_FINALIZER(G, remove_SchreyerOrder, 0, 0, 0);
+#ifdef MEMDEBUG
+  GC_REGISTER_FINALIZER(M2_debug_to_outer(G), remove_SchreyerOrder, 0, 0, 0);
+#else
+  GC_REGISTER_FINALIZER(                  G , remove_SchreyerOrder, 0, 0, 0);
+#endif
   AO_t nfinalized = AO_fetch_and_add1(&schorder_nfinalized);
   if (M2_gbTrace >= 3)
     fprintf(stderr,
@@ -136,6 +173,9 @@ void intern_SchreyerOrder(SchreyerOrder *G)
 
 extern "C" void remove_MutableMatrix(void *p, void *cd)
 {
+#ifdef MEMDEBUG
+  p = M2_debug_to_inner(p);
+#endif
   MutableMatrix *G = static_cast<MutableMatrix *>(p);
   AO_t nremoved = AO_fetch_and_add1(&mutablematrices_nremoved);
   if (M2_gbTrace >= 3)
@@ -145,7 +185,11 @@ extern "C" void remove_MutableMatrix(void *p, void *cd)
 MutableMatrix *internMutableMatrix(MutableMatrix *G)
 {
   if (G == 0) return 0;
-  GC_REGISTER_FINALIZER(G, remove_MutableMatrix, 0, 0, 0);
+#ifdef MEMDEBUG
+  GC_REGISTER_FINALIZER(M2_debug_to_outer(G), remove_MutableMatrix, 0, 0, 0);
+#else
+  GC_REGISTER_FINALIZER(                  G,  remove_MutableMatrix, 0, 0, 0);
+#endif
   AO_t nfinalized = AO_fetch_and_add1(&mutablematrices_nfinalized);
   if (M2_gbTrace >= 3)
     fprintf(stderr,
