@@ -15,10 +15,17 @@
 #include "exceptions.hpp"
 #include "gb-walk.hpp"
 #include "relem.hpp"
+#include "util.hpp"
+#include "matrix-ncbasis.hpp"
+
+#include "M2FreeAlgebra.hpp"
+#include "NCAlgebras/FreeAlgebra.hpp"
+#include "NCAlgebras/NCGroebner.hpp"
+#include "NCAlgebras/NCF4.hpp"
 
 #include "poly.hpp"
 #include "interrupted.hpp"
-#include "f4/res-f4-computation.hpp"
+#include "schreyer-resolution/res-f4-computation.hpp"
 
 class FreeModule;
 struct MonomialOrdering;
@@ -45,388 +52,12 @@ void test_over_RR_or_CC(const Ring *R)
         }
     }
 }
-////////////////////////////////////
-// new GB computations /////////////
-////////////////////////////////////
-EngineComputationOrNull *rawGB(
-    const Matrix *m,
-    M2_bool collect_syz,
-    int n_rows_to_keep,
-    M2_arrayint gb_weights,
-    M2_bool use_max_degree,
-    int max_degree,
-    int algorithm,
-    int strategy,
-    int max_reduction_count) /* drg: connected rawGB */
-{
-  // Choose the correct computation here.
-  try
-    {
-      clear_emit_size();
-      return EngineGBComputation::create(
-          GBBComputation::choose_gb(m,
-                                    collect_syz,
-                                    n_rows_to_keep,
-                                    gb_weights,
-                                    use_max_degree,
-                                    max_degree,
-                                    algorithm,
-                                    strategy,
-                                    max_reduction_count));
-  } catch (const exc::engine_error& e)
-    {
-      ERROR(e.what());
-      return NULL;
-  }
-}
-
-EngineComputationOrNull *rawGBSetHilbertFunction(EngineComputation *C,
-                                                 const RingElement *h)
-{
-  try
-    {
-      clear_emit_size();
-      EngineGBComputation *G = C->cast_to_EngineGBComputation();
-      if (G != 0) return G->set_hilbert_function(h);
-      ERROR("computation type unknown or not implemented");
-      return 0;
-  } catch (const exc::engine_error& e)
-    {
-      ERROR(e.what());
-      return NULL;
-  }
-}
-
-void rawComputationSetStop(EngineComputation *G,
-                           M2_bool always_stop,
-                           M2_arrayint degree_limit,
-                           int basis_element_limit,
-                           int syzygy_limit,
-                           int pair_limit,
-                           int codim_limit,
-                           int subring_limit,
-                           M2_bool just_min_gens,
-                           M2_arrayint length_limit)
-/* LongPolynomial, Sort, Primary, Inhomogeneous, Homogeneous */
-/* Res: SortStrategy, 0, 1, 2, 3 ?? */
-{
-  // No errors can surface here.
-  clear_emit_size();
-  G->set_stop_conditions(always_stop,
-                         degree_limit,
-                         basis_element_limit,
-                         syzygy_limit,
-                         pair_limit,
-                         codim_limit,
-                         subring_limit,
-                         just_min_gens,
-                         length_limit);
-}
-
-EngineComputationOrNull *rawStartEngineComputation(EngineComputation *C)
-/* start or continue the computation */
-{
-  try
-    {
-      clear_emit_size();
-      C->start_computation();
-
-      if (M2_gbTrace == 15)
-        {
-          ComputationStatusCode ret = C->status();
-          switch (ret)
-            {
-              case COMP_DONE_DEGREE_LIMIT:
-                emit_line("computation stopped at degree limit");
-                break;
-              case COMP_DONE:
-                emit_line("computation of GB completed");
-                break;
-              case COMP_DONE_PAIR_LIMIT:
-                emit_line("computation stopped at pair limit");
-                break;
-              case COMP_NEED_RESIZE:
-              case COMP_ERROR:
-              case COMP_INTERRUPTED:
-              case COMP_NOT_STARTED:
-              case COMP_INITIAL_STOP:
-              case COMP_DONE_LENGTH_LIMIT:
-              case COMP_DONE_SYZYGY_LIMIT:
-              case COMP_DONE_GB_LIMIT:
-              case COMP_DONE_SYZ_LIMIT:
-              case COMP_DONE_CODIM:
-              case COMP_DONE_MIN_GENS:
-              case COMP_DONE_STEPS:
-              case COMP_DONE_SUBRING_LIMIT:
-              case COMP_COMPUTING:
-              case COMP_OVERFLOWED:
-                emit_line("computation stopped for some good reason");
-                break;
-              default:
-                emit_line("incorrect status code encountered");
-                break;
-            }
-        }
-      return error() ? 0 : C;
-  } catch (const exc::engine_error& e)
-    {
-      ERROR(e.what());
-      return NULL;
-  }
-}
-
-enum ComputationStatusCode rawEngineStatus1(EngineComputation *C)
-{
-  return C->status();
-}
-
-long rawEngineStatus2(EngineComputation *C)
-{
-  return C->complete_thru_degree();
-}
-
-void rawShowEngineComputation(const EngineComputation *C) { C->show(); }
-const Matrix /* or null */ *rawEngineGBGetMatrix(EngineComputation *C)
-/* Get the minimal, auto-reduced GB of a GB computation.
-   Each call to this will produce a different raw matrix */
-{
-  try
-    {
-      clear_emit_size();
-      EngineGBComputation *G = C->cast_to_EngineGBComputation();
-      if (G != 0) return G->get_GroebnerBasis()->get_gb();
-      ERROR("computation type unknown or not implemented");
-      return 0;
-  } catch (const exc::engine_error& e)
-    {
-      ERROR(e.what());
-      return NULL;
-  }
-}
-
-const Matrix /* or null */ *rawEngineGBMinimalGenerators(EngineComputation *C)
-/* Yields a matrix whose columns form a minimal generating set
-   for the ideal or submodule, as computed so far.  In the
-   inhomogeneous case, this yields a generating set which is
-   sometimes smaller than the entire Groebner basis. */
-{
-  try
-    {
-      clear_emit_size();
-      EngineGBComputation *G = C->cast_to_EngineGBComputation();
-      if (G != 0) return G->get_GroebnerBasis()->get_mingens();
-      ERROR("computation type unknown or not implemented");
-      return 0;
-  } catch (const exc::engine_error& e)
-    {
-      ERROR(e.what());
-      return NULL;
-  }
-}
-
-const Matrix /* or null */ *rawEngineGBChangeOfBasis(EngineComputation *C)
-/* Yields the change of basis matrix from the Groebner basis to
-   the original generators, at least if n_rows_to_keep was set
-   when creating the GB computation.  This matrix, after the
-   computation has run to completion, should satisfy:
-   (original matrix) = (GB matrix) * (change of basis matrix). */
-{
-  try
-    {
-      clear_emit_size();
-      EngineGBComputation *G = C->cast_to_EngineGBComputation();
-      if (G != 0) return G->get_GroebnerBasis()->get_change();
-      ERROR("computation type unknown or not implemented");
-      return 0;
-  } catch (const exc::engine_error& e)
-    {
-      ERROR(e.what());
-      return NULL;
-  }
-}
-
-const Matrix /* or null */ *rawEngineGBGetLeadTerms(EngineComputation *C,
-                                                    int nparts)
-{
-  try
-    {
-      clear_emit_size();
-      EngineGBComputation *G = C->cast_to_EngineGBComputation();
-      if (G != 0) return G->get_GroebnerBasis()->get_initial(nparts);
-      ERROR("computation type unknown or not implemented");
-      return 0;
-  } catch (const exc::engine_error& e)
-    {
-      ERROR(e.what());
-      return NULL;
-  }
-}
-
-const Matrix /* or null */ *rawEngineGBGetParallelLeadTerms(
-    EngineComputation *C,
-    M2_arrayint w)
-{
-  try
-    {
-      clear_emit_size();
-      EngineGBComputation *G = C->cast_to_EngineGBComputation();
-      if (G != 0) return G->get_GroebnerBasis()->get_parallel_lead_terms(w);
-      ERROR("computation type unknown or not implemented");
-      return 0;
-  } catch (const exc::engine_error& e)
-    {
-      ERROR(e.what());
-      return NULL;
-  }
-}
-
-const Matrix /* or null */ *rawEngineGBSyzygies(EngineComputation *C)
-/* Yields a matrix containing the syzygies computed so far
-   via the GB computation C, assuming that 'collect_syz' was
-   set when the computation was created.  If 'n_rows_to_keep' was
-   set to a non-negative integer, then only that many rows of each
-   syzygy are kept. */
-{
-  try
-    {
-      clear_emit_size();
-      EngineGBComputation *G = C->cast_to_EngineGBComputation();
-      if (G != 0) return G->get_GroebnerBasis()->get_syzygies();
-      ERROR("computation type unknown or not implemented");
-      return 0;
-  } catch (const exc::engine_error& e)
-    {
-      ERROR(e.what());
-      return NULL;
-  }
-}
-
-const Matrix /* or null */ *rawEngineGBMatrixRemainder(EngineComputation *C,
-                                                       const Matrix *m)
-{
-  try
-    {
-      clear_emit_size();
-      EngineGBComputation *G = C->cast_to_EngineGBComputation();
-      if (G != 0) return G->get_GroebnerBasis()->matrix_remainder(m);
-      ERROR("computation type unknown or not implemented");
-      return 0;
-  } catch (const exc::engine_error& e)
-    {
-      ERROR(e.what());
-      return NULL;
-  }
-}
-
-void rawEngineGBMatrixDivMod(EngineComputation *C,
-                             const Matrix *m,
-                             const Matrix /* or null */ **result_remainder,
-                             const Matrix /* or null */ **result_quotient)
-{
-  try
-    {
-      clear_emit_size();
-      EngineGBComputation *G = C->cast_to_EngineGBComputation();
-      if (G != 0)
-        G->get_GroebnerBasis()->matrix_lift(
-            m, result_remainder, result_quotient);
-      else
-        ERROR("computation type unknown or not implemented");
-  } catch (const exc::engine_error& e)
-    {
-      ERROR(e.what());
-      return;
-  }
-}
-
-int rawEngineGBMatrixContains(EngineComputation *C, const Matrix *m)
-{
-  try
-    {
-      clear_emit_size();
-      EngineGBComputation *G = C->cast_to_EngineGBComputation();
-      if (G != 0) return G->get_GroebnerBasis()->contains(m);
-      ERROR("computation type unknown or not implemented");
-      return -2;
-  } catch (const exc::engine_error& e)
-    {
-      ERROR(e.what());
-      return -2;
-  }
-}
-
-EngineComputationOrNull *rawEngineGBDeclared(
-    const Matrix *m, /* trimmed or minimal gens, may be the same as gb */
-    const Matrix *gb,
-    const Matrix *change, /* same number of columns as 'gb', if not 0 */
-    const Matrix *syz)    /* possibly 0 too, otherwise same rows as change */
-{
-  try
-    {
-      return EngineGBComputation::create(
-          GBDeclared::create(m, gb, change, syz));
-  } catch (const exc::engine_error& e)
-    {
-      ERROR(e.what());
-      return NULL;
-  }
-}
-
-EngineComputationOrNull *rawMarkedEngineGB(
-    const Matrix *leadterms,
-    const Matrix *m, /* trimmed or minimal gens, may be the same as gb */
-    const Matrix *gb,
-    const Matrix *change, /* same number of columns as 'gb', if not 0 */
-    const Matrix *syz)    /* possibly 0 too, otherwise same rows as change */
-{
-  try
-    {
-      return EngineGBComputation::create(
-          GBDeclared::create(leadterms, m, gb, change, syz));
-  } catch (const exc::engine_error& e)
-    {
-      ERROR(e.what());
-      return NULL;
-  }
-}
-
-EngineComputationOrNull *rawEngineGroebnerWalk(const Matrix *gb,
-                                               const MonomialOrdering *order1)
-{
-  try
-    {
-      return EngineGBComputation::create(GBWalker::create(gb, order1));
-  } catch (const exc::engine_error& e)
-    {
-      ERROR(e.what());
-      return NULL;
-  }
-}
-
-M2_string rawEngineComputationToString(EngineComputation *C)
-{
-  buffer o;
-  try
-    {
-      C->text_out(o);
-      return o.to_string();
-  } catch (const exc::engine_error& e)
-    {
-      o << "[unprintable computation]";
-      return o.to_string();
-  }
-}
-
-unsigned long rawEngineComputationHash(const Computation *C)
-{
-  return C->hash();
-}
 
 ////////////////////////////////////
 const RingElement /* or null */ *IM2_Matrix_Hilbert(const Matrix *M)
 /* This routine computes the numerator of the Hilbert series
    for coker leadterms(M), using the degrees of the rows of M.
-   NULL is returned if the ring is not appropriate for
+   nullptr is returned if the ring is not appropriate for
    computing Hilbert series, or the computation was interrupted. */
 {
   try
@@ -435,7 +66,7 @@ const RingElement /* or null */ *IM2_Matrix_Hilbert(const Matrix *M)
   } catch (const exc::engine_error& e)
     {
       ERROR(e.what());
-      return NULL;
+      return nullptr;
   }
 }
 
@@ -471,7 +102,7 @@ Computation /* or null */ *IM2_GB_make(
   } catch (const exc::engine_error& e)
     {
       ERROR(e.what());
-      return NULL;
+      return nullptr;
   }
 }
 
@@ -498,7 +129,7 @@ Computation /* or null */ *IM2_res_make(const Matrix *m,
   } catch (const exc::engine_error& e)
     {
       ERROR(e.what());
-      return NULL;
+      return nullptr;
   }
 }
 
@@ -520,7 +151,7 @@ Computation /* or null */ *IM2_GB_set_hilbert_function(Computation *C,
   } catch (const exc::engine_error& e)
     {
       ERROR(e.what());
-      return NULL;
+      return nullptr;
   }
 }
 
@@ -537,7 +168,7 @@ Computation /* or null */ *IM2_GB_force(
   } catch (const exc::engine_error& e)
     {
       ERROR(e.what());
-      return NULL;
+      return nullptr;
   }
 }
 
@@ -555,7 +186,7 @@ Computation /* or null */ *rawMarkedGB(
   } catch (const exc::engine_error& e)
     {
       ERROR(e.what());
-      return NULL;
+      return nullptr;
   }
 }
 
@@ -568,7 +199,7 @@ Computation /* or null */ *rawGroebnerWalk(const Matrix *gb,
   } catch (const exc::engine_error& e)
     {
       ERROR(e.what());
-      return NULL;
+      return nullptr;
   }
 }
 
@@ -601,7 +232,7 @@ Computation /* or null */ *IM2_Computation_set_stop(
   } catch (const exc::engine_error& e)
     {
       ERROR(e.what());
-      return NULL;
+      return nullptr;
   }
 }
 
@@ -654,7 +285,7 @@ Computation /* or null */ *rawStartComputation(Computation *C)
   } catch (const exc::engine_error& e)
     {
       ERROR(e.what());
-      return NULL;
+      return nullptr;
   }
 }
 
@@ -675,7 +306,7 @@ const Matrix /* or null */ *rawGBGetMatrix(Computation *C)
   } catch (const exc::engine_error& e)
     {
       ERROR(e.what());
-      return NULL;
+      return nullptr;
   }
 }
 
@@ -695,7 +326,7 @@ const Matrix /* or null */ *rawGBMinimalGenerators(Computation *C)
   } catch (const exc::engine_error& e)
     {
       ERROR(e.what());
-      return NULL;
+      return nullptr;
   }
 }
 
@@ -716,7 +347,7 @@ const Matrix /* or null */ *rawGBChangeOfBasis(Computation *C)
   } catch (const exc::engine_error& e)
     {
       ERROR(e.what());
-      return NULL;
+      return nullptr;
   }
 }
 
@@ -726,13 +357,13 @@ const Matrix /* or null */ *rawGBGetLeadTerms(Computation *C, int nparts)
     {
       clear_emit_size();
       GBComputation *G = C->cast_to_GBComputation();
-      if (G != 0) return G->get_initial(nparts);
+      if (G != nullptr) return G->get_initial(nparts);
       ERROR("computation type unknown or not implemented");
-      return 0;
+      return nullptr;
   } catch (const exc::engine_error& e)
     {
       ERROR(e.what());
-      return NULL;
+      return nullptr;
   }
 }
 
@@ -749,7 +380,7 @@ const Matrix /* or null */ *rawGBGetParallelLeadTerms(Computation *C,
   } catch (const exc::engine_error& e)
     {
       ERROR(e.what());
-      return NULL;
+      return nullptr;
   }
 }
 
@@ -770,7 +401,7 @@ const Matrix /* or null */ *rawGBSyzygies(Computation *C)
   } catch (const exc::engine_error& e)
     {
       ERROR(e.what());
-      return NULL;
+      return nullptr;
   }
 }
 
@@ -787,7 +418,7 @@ const Matrix /* or null */ *rawGBMatrixRemainder(Computation *C,
   } catch (const exc::engine_error& e)
     {
       ERROR(e.what());
-      return NULL;
+      return nullptr;
   }
 }
 
@@ -839,7 +470,7 @@ const Matrix /* or null */ *rawResolutionGetMatrix(Computation *C, int level)
   } catch (const exc::engine_error& e)
     {
       ERROR(e.what());
-      return NULL;
+      return nullptr;
   }
 }
 
@@ -857,7 +488,7 @@ MutableMatrix /* or null */ *rawResolutionGetMatrix2(Computation *C,
   } catch (const exc::engine_error& e)
     {
       ERROR(e.what());
-      return NULL;
+      return nullptr;
   }
 }
 
@@ -880,7 +511,7 @@ MutableMatrix /* or null */ *rawResolutionGetMutableMatrixB(Computation *C,
   } catch (const exc::engine_error& e)
     {
       ERROR(e.what());
-      return NULL;
+      return nullptr;
   }
 }
 
@@ -900,7 +531,7 @@ MutableMatrix /* or null */ *rawResolutionGetMutableMatrix2B(
   } catch (const exc::engine_error& e)
     {
       ERROR(e.what());
-      return NULL;
+      return nullptr;
   }
 }
 
@@ -916,7 +547,7 @@ const FreeModule /* or null */ *rawResolutionGetFree(Computation *C, int level)
   } catch (const exc::engine_error& e)
     {
       ERROR(e.what());
-      return NULL;
+      return nullptr;
   }
 }
 
@@ -959,11 +590,11 @@ M2_arrayintOrNull rawResolutionBetti(Computation *C, int type)
       ResolutionComputation *G = C->cast_to_ResolutionComputation();
       if (G != 0) return G->get_betti(type);
       ERROR("expected resolution computation type");
-      return NULL;
+      return nullptr;
   } catch (const exc::engine_error& e)
     {
       ERROR(e.what());
-      return NULL;
+      return nullptr;
   }
 }
 
@@ -999,7 +630,7 @@ Matrix /* or null */ *rawSubduction(int numparts, const Matrix *M,
   } catch (const exc::engine_error& e)
     {
       ERROR(e.what());
-      return NULL;
+      return nullptr;
   }
 }
 
@@ -1190,10 +821,147 @@ const Matrix *rawMGB(
   } catch (const std::runtime_error& e)
     {
       ERROR(e.what());
-      return NULL;
+      return nullptr;
   }
 }
 
+/////////////////////////////////////////////
+// Noncommutative Groebner bases (2-sided) //
+/////////////////////////////////////////////
+
+ConstPolyList matrixToPolyList(const M2FreeAlgebraOrQuotient* A,
+                               const Matrix* input)
+{
+  ConstPolyList result;
+  result.reserve(input->n_cols() * input->n_rows());
+  for (int i=0; i < input->n_rows(); i++)
+    {
+      for (int j=0; j < input->n_cols(); ++j)
+      {
+        ring_elem a = input->elem(i,j);
+        auto f = reinterpret_cast<const Poly*>(a.get_Poly());
+        result.push_back(f);
+      }
+    }
+  return result;
+}
+
+// vectorToMatrix consumes 'elems': the same pointers are used for the resulting Matrix.
+template<typename PolyL>
+const Matrix* polyListToMatrix(const M2FreeAlgebraOrQuotient* A,
+                               const PolyL& elems,
+                               int numrows,
+                               int numcols)
+{
+  if (elems.size() != numrows*numcols)
+    ERROR("Number of elements in list does not match matrix size.");
+  MatrixConstructor mat(A->make_FreeModule(numrows), numcols);
+  for (auto i = 0; i < elems.size(); ++i)
+    {
+      int curCol = i % numcols;
+      int curRow = (i - curCol) / numcols;
+      ring_elem a = const_cast<Nterm*>(reinterpret_cast<const Nterm*>(elems[i]));
+      mat.set_entry(curRow, curCol, a);
+    }
+  mat.compute_column_degrees();
+  return mat.to_matrix();
+}
+
+const Matrix* rawNCGroebnerBasisTwoSided(const Matrix* input, int maxdeg, int strategy)
+{
+  const Ring* R = input->get_ring();
+  const M2FreeAlgebra* A = R->cast_to_M2FreeAlgebra();
+  if (A != nullptr and input->n_rows() == 1)
+    {
+      auto elems = matrixToPolyList(A, input);
+      bool isF4 = strategy & 16;
+      bool isParallel = strategy & 32;
+      if (isF4)
+        {
+          NCF4 G(A->freeAlgebra(), elems, maxdeg, strategy, isParallel);
+          G.compute(maxdeg); // this argument is actually the soft degree limit
+          auto result = copyPolyVector(A, G.currentValue());
+          return polyListToMatrix(A, result, 1, result.size()); // consumes the Poly's in result
+        }
+      else
+        {
+          NCGroebner G(A->freeAlgebra(), elems, maxdeg, strategy);
+          G.compute(maxdeg); // this argument is actually the soft degree limit
+          auto result = copyPolyVector(A, G.currentValue());
+          return polyListToMatrix(A, result, 1, result.size()); // consumes the Poly's in result
+        }
+
+    }
+  ERROR("expected a one row matrix over a noncommutative algebra");
+  return nullptr;
+}
+
+const Matrix* rawNCReductionTwoSided(const Matrix* toBeReduced, const Matrix* reducerMatrix)
+{
+  const Ring* R = toBeReduced->get_ring();
+  if (R != reducerMatrix->get_ring())
+    {
+      ERROR("expected matrices to be over the same ring");
+      return nullptr;
+    }
+  const M2FreeAlgebra* A = R->cast_to_M2FreeAlgebra();
+  if (A != nullptr and reducerMatrix->n_rows() == 1)
+    {
+      auto outRows = toBeReduced->n_rows();
+      auto outCols = toBeReduced->n_cols();
+      auto reducees = matrixToPolyList(A, toBeReduced);
+      auto reducers = matrixToPolyList(A, reducerMatrix);
+      NCGroebner G(A->freeAlgebra(),reducers, 0, 0);
+      G.initReductionOnly();
+      auto result = G.twoSidedReduction(reducees);
+      return polyListToMatrix(A, result, outRows, outCols); // consumes the Poly's in result.
+    }
+  ERROR("expected a matrix over a noncommutative algebra");
+  return nullptr;
+}
+
+const Matrix* rawNCBasis(const Matrix* gb2SidedIdeal,
+                         M2_arrayint lo_degree,
+                         M2_arrayint hi_degree,
+                         int limit
+                         )
+{
+  const Ring* R = gb2SidedIdeal->get_ring();
+  if (R == nullptr)
+    {
+      ERROR("internal error: expected non-null Ring!");
+      return nullptr;
+    }
+  try {
+    const M2FreeAlgebra* A = R->cast_to_M2FreeAlgebra();
+    if (A != nullptr)
+      {
+        ConstPolyList G = matrixToPolyList(A, gb2SidedIdeal);
+
+        // WARNING: The following line creates new polynomials
+        // which are used directly in vectorToMatrix (without copying)
+        // but when result goes out of scope, the list is deleted,
+        // but not the polynomials themselves, as they are pointers.
+        PolyList result;
+        bool worked = ncBasis(A->freeAlgebra(),
+                              G,
+                              M2_arrayint_to_stdvector<int>(lo_degree),
+                              M2_arrayint_to_stdvector<int>(hi_degree),
+                              limit,
+                              result);
+        if (not worked) return nullptr;
+        return polyListToMatrix(A, result, 1, result.size()); // consumes entries of result
+      }
+    ERROR("expected a free algebra");
+    return nullptr;
+  }
+  catch (exc::engine_error& e) {
+    ERROR(e.what());
+    return nullptr;
+  }
+}
+
+
 // Local Variables:
 // compile-command: "make -C $M2BUILDDIR/Macaulay2/e "
 // indent-tabs-mode: nil
